######################################################
#                TkTeXEditor
#        Distributed under GNU Public License
# Author: Sergey Kalinin (BanZaj) banzaj@lrn.ru
# Copyright (c) "CONERO lab", 2002, http://conero.lrn.ru
######################################################

######################################################
#                                                    #
# Document structure module (insert\update\deleted   #
#                                                    #
######################################################

set in(c) 0 ;# chapter counter
set in(p) 0 ;# part counter
set in(s) 0 ;# section counter
set in(ss) 0 ;# subsection counter
set in(sss) 0 ;# subsubsection counter
set in(par) 0 ;# paragraph cponter
set in(spar) 0 ;# subparagraph cponter
set in(lbl) 0 ;# label cuonter
set in(ref) 0 ;# ref counter
set in(pageref) 0 ;# ref counter
set in(include) 0
set in(input) 0
set in(cite) 0
set in(bibitem) 0
set lastPart "root"
set lastChapter "root"
set lastSection "root"
set lastSSection "root"
set lastSSSection "root"
set lastPar "root"
set lastSubPar "root"

## INSERT STRUCT PROCEDURE ##
## reading and parsing string from surce file
## and execute procedures for inserted required structure into structure tree
proc InsertStruct {line lineNumber fileDir} {
    global tree treeStruct dir font color in lastNode activeFile
    global lastPart lastChapter lastSection lastSSection lastSSSection
    global lastPar lastSubPar lastLabel
    
    if [regexp -nocase -all -- {(\\part)(\*|)(\[.*?\])*(\{.*?\})} $line struct v1 v1_5 v2 v3] {
        incr in(p)
        InsertNode "part_$in(p)" "root" "[string trimleft [string trimright $v1_5$v3 "\}"] "\{"]" part $lineNumber        
        set lastPart "part_$in(p)"
    }
    if [regexp -nocase -all -- {(\\chapter)(\*|)(\[.*?\])*(\{.*?\})} $line struct v1 v1_5 v2 v3] {
        incr in(c)
        InsertNode "chapter_$in(c)" "$lastPart" "[string trimleft [string trimright $v1_5$v3 "\}"] "\{"]" chapter $lineNumber
        set lastChapter chapter_$in(c)
    }
    if [regexp -nocase -all -- {(\\section)(\*|)(\[.*?\])*(\{.*?\})} $line struct v1 v1_5 v2 v3] {
        incr in(s)
        InsertNode "section_$in(s)" "$lastChapter" "[string trimleft [string trimright $v1_5$v3 "\}"] "\{"]" section $lineNumber
        set lastSection "section_$in(s)"
    }
    if [regexp -nocase -all -- {(\\subsection)(\*|)(\[.*?\])*(\{.*?\})} $line struct v1 v1_5 v2 v3] {
        incr in(ss)
        InsertNode "subsection_$in(ss)" "$lastSection" "[string trimleft [string trimright $v1_5$v3 "\}"] "\{"]" ssection $lineNumber
        set lastSSection "subsection_$in(ss)"
    }
    if [regexp -nocase -all -- {(\\subsubsection)(\*|)(\[.*?\])*(\{.*?\})} $line struct v1 v1_5 v2 v3] {
        incr in(sss)
        InsertNode "subsubsection_$in(sss)" "$lastSSection" "[string trimleft [string trimright $v1_5$v3 "\}"] "\{"]" sssection $lineNumber
        set lastSSSection "subsubsection_$in(sss)"
    }
    if [regexp -nocase -all -- {(\\paragraph)(\*|)(\[.*?\])*(\{.*?\})} $line struct v1 v1_5 v2 v3] {
        incr in(par)
        InsertNode "paragraph_$in(par)" "$lastSSSection" "[string trimleft [string trimright $v1_5$v3 "\}"] "\{"]" paragraph $lineNumber
        set lastPar "paragraph_$in(par)"
    }
    if [regexp -nocase -all -- {(\\subparagraph)(\*|)(\[.*?\])*(\{.*?\})} $line struct v1 v1_5 v2 v3] {
        incr in(spar)
        InsertNode "subparagraph_$in(spar)" "$lastPar" "[string trimleft [string trimright $v1_5$v3 "\}"] "\{"]" subparagraph $lineNumber
        set lastSubPar "subparagraph_$in(spar)"
    }
    if [regexp -nocase -all -- {(\\label)(\{.*?\})} $line struct v1 v2] {
        incr in(lbl)
        InsertLabel "label_$in(lbl)" "$v2" "label"
    }
    if [regexp -nocase -all -- {(\\ref)(\{.*?\})} $line struct v1 v2] {
        incr in(ref)
        InsertRef "ref_$in(ref)" "$v2"
    }
    if [regexp -nocase -all -- {(\\pageref)(\{.*?\})} $line struct v1 v2] {
        incr in(pageref)
        InsertPageref "pageref_$in(pageref)" "$v2"
    }
    if [regexp -nocase -all -- {(\\cite)(\{.*?\})} $line struct v1 v2] {
        incr in(cite)
        InsertLabel "cite_$in(cite)" "$v2" "cite"
    }
    if [regexp -nocase -all -- {(\\bibitem)(\{.*?\})} $line struct v1 v2] {
        incr in(bibitem)
        InsertLabel "bibitem_$in(bibitem)" "$v2" "bibitem"
    }
    if [regexp -nocase -all -- {(\\input)(\{.*?\})} $line struct v1 v2] {
        incr in(input)
        set fullPath [string trimleft [string trimright $v2 "\}"] "\{"]
        set dirName [file dirname $fullPath]
        if {$dirName != "."} {
            set fileDir [file join $fileDir $dirName]
        }
        set file [file tail $fullPath]
        InsertTreeSubNode "$file" "$activeFile" $fileDir
    }
    if [regexp -nocase -all -- {(\\include)(\{.*?\})} $line struct v1 v2] {
        incr in(include)
        set fullPath [string trimleft [string trimright $v2 "\}"] "\{"]
        set dirName [file dirname $fullPath]
        if {$dirName != "."} {
            set fileDir [file join $fileDir $dirName]
        }
        set file [file tail $fullPath]
        InsertTreeSubNode "$file" "$activeFile" $fileDir
    }
}
## UPDATE DOCUMENT STRUCTURE PROCEDURE ##
proc UpdateStruct {file} {
    global tree treeStruct dir font color  lblList files
    global lastPart lastChapter lastSection lastSSection lastSSSection
    global lastPar lastSubPar in nb dir activeFile
    if [info exists files($activeFile)] {
        Timer $file "refresh"
    } else {
        return
    }
    #FileDialog save
    #        
    set dirName [file dirname $file]
    set file [file tail $file]
    set text "$nb.f$activeFile.f.text"
    set contents [$text get 0.0 end]
    set fhandle [open [file join $dir(tmp) $file.bak] "w"]
    puts $fhandle $contents nonewline
    close $fhandle
    
    
    set in(c) 0 ;# chapter counter
    set in(p) 0 ;# part counter
    set in(s) 0 ;# section counter
    set in(ss) 0 ;# subsection counter
    set in(sss) 0 ;# subsubsection counter
    set in(par) 0 ;# paragraph cponter
    set in(spar) 0 ;# subparagraph cponter
    set in(lbl) 0 ;# label cuonter
    set in(ref) 0 ;# ref counter
    set in(pageref) 0 ;# ref counter
    set in(include) 0
    set in(input) 0
    set in(cite) 0
    set in(bibitem) 0
    set lastPart "root"
    set lastChapter "root"
    set lastSection "root"
    set lastSSection "root"
    set lastSSSection "root"
    set lastPar "root"
    set lastSubPar "root"
    ## check if file was deleted from declaration \input and \include
    ## adding new node into file tree
    if [$tree exists $activeFile] {
        $tree delete [$tree nodes $activeFile] ;# delete all info about old structure
    } else {
        set file [file tail [lindex $files($activeFile) 0]]
        set fileDir [file dirname [lindex $files($activeFile) 0]]
        InsertTreeNode "$file" $fileDir
    }
    $treeStruct delete [$treeStruct nodes root] ;# delete all info about old structure
    $lblList delete [$lblList item 0 end] ;# deleted all old labels from label list
    set fHandle [open [file join $dir(tmp) $file.bak] "r"]
    set lineNumber 1
    while {[gets $fHandle line]>=0} {
        InsertStruct $line $lineNumber $dirName
        incr lineNumber
    }
    unset dirName text contents 
}

proc InsertLabel {node label img} {
    global tree treeStruct  lblList dir
    set label [string trimleft $label "\{"]
    set label [string trimright $label "\}"]
    $lblList insert end $node -text "$label" -image [Bitmap::get [file join $dir(img) $img.gif]]
    unset label
}
proc InsertRef {node label} {
    global tree treeStruct  lblList dir
    set label [string trimleft $label "\{"]
    set label [string trimright $label "\}"]
    $lblList insert end $node -text "$label" -image [Bitmap::get [file join $dir(img) ref.gif]]
    unset label
}
proc InsertPageref {node label} {
    global tree treeStruct lblList dir
    set label [string trimleft $label "\{"]
    set label [string trimright $label "\}"]
    $lblList insert end $node -text "$label" -image [Bitmap::get [file join $dir(img) pageref.gif]]
    unset label
}
proc FindLabel {node} {
    global nb activeFile lblList status
    $lblList selection set $node        
    set t [$lblList itemcget $node -text]
    set text [string range $node 0 [expr [string first "_" $node]-1]]
    set findString "$text\{$t\}"
    set text "$nb.f$activeFile.f.text"
    FindProc $text $findString $activeFile
    $status(pos) configure -text [$text index insert];# cursor position
    unset text t findString
}







